@@ -62,13 +62,9 @@ import sys
 import hashlib
 import logging
 import datetime
-if sys.version[0] == '2':
-    import Queue
-    import thread
-else:
-    import queue as  Queue
-    import _thread as thread
-
+from six import reraise
+from six.moves import queue as Queue
+from six.moves import _thread as thread
 from xml.etree import ElementTree
 from bmaptools.BmapHelpers import human_size
 
@@ -495,6 +491,7 @@ class BmapCopy(object):
           * 'buf' a buffer containing the batch data.
         """
 
+        _log.debug("the reader thread has started")
         try:
             for (first, last, chksum) in self._get_block_ranges():
                 if verify and chksum:
@@ -512,6 +509,8 @@ class BmapCopy(object):
                                     % (start, end, self._image_path, err))
 
                     if not buf:
+                        _log.debug("no more data to read from file '%s'",
+                                   self._image_path)
                         self._batch_queue.put(None)
                         return
 
@@ -581,7 +580,7 @@ class BmapCopy(object):
                 # The reader thread encountered an error and passed us the
                 # exception.
                 exc_info = batch[1]
-                raise exc_info[1].with_traceback(exc_info[2])
+                reraise(exc_info[0], exc_info[1], exc_info[2])
 
             (start, end, buf) = batch[1:4]
 
@@ -723,9 +722,9 @@ class BmapBdevCopy(BmapCopy):
                 f_scheduler.seek(0)
                 f_scheduler.write("noop")
         except IOError as err:
-            _log.warning("failed to enable I/O optimization, expect "
+            _log.debug("failed to enable I/O optimization, expect "
                          "suboptimal speed (reason: cannot switch to the "
-                         "'noop' I/O scheduler: %s)" % err)
+                         "'noop' I/O scheduler: %s or blk-mq in use)" % err)
         else:
             # The file contains a list of schedulers with the current
             # scheduler in square brackets, e.g., "noop deadline [cfq]".
