@@ -28,6 +28,7 @@ and returns an instance of the class.
 # pylint: disable=R0902
 
 import os
+import errno
 import struct
 import array
 import fcntl
@@ -176,9 +177,9 @@ def _lseek(file_obj, offset, whence):
     except OSError as err:
         # The 'lseek' system call returns the ENXIO if there is no data or
         # hole starting from the specified offset.
-        if err.errno == os.errno.ENXIO:
+        if err.errno == errno.ENXIO:
             return -1
-        elif err.errno == os.errno.EINVAL:
+        elif err.errno == errno.EINVAL:
             raise ErrorNotSupp("the kernel or file-system does not support "
                                "\"SEEK_HOLE\" and \"SEEK_DATA\"")
         else:
@@ -219,7 +220,7 @@ class FilemapSeek(_FilemapBase):
 
         try:
             tmp_obj = tempfile.TemporaryFile("w+", dir=directory)
-        except IOError as err:
+        except OSError as err:
             raise ErrorNotSupp("cannot create a temporary in \"%s\": %s"
                                % (directory, err))
 
@@ -383,12 +384,12 @@ class FilemapFiemap(_FilemapBase):
         except IOError as err:
             # Note, the FIEMAP ioctl is supported by the Linux kernel starting
             # from version 2.6.28 (year 2008).
-            if err.errno == os.errno.EOPNOTSUPP:
+            if err.errno == errno.EOPNOTSUPP:
                 errstr = "FilemapFiemap: the FIEMAP ioctl is not supported " \
                          "by the file-system"
                 _log.debug(errstr)
                 raise ErrorNotSupp(errstr)
-            if err.errno == os.errno.ENOTTY:
+            if err.errno == errno.ENOTTY:
                 errstr = "FilemapFiemap: the FIEMAP ioctl is not supported " \
                          "by the kernel"
                 _log.debug(errstr)
