@@ -16,7 +16,7 @@
 
 """
 This module allows opening and reading local and remote files and decompress
-them on-the-fly if needed. Remote files are read using urllib2 (except of
+them on-the-fly if needed. Remote files are read using urllib (except of
 "ssh://" URLs, which are handled differently). Supported file extentions are:
 'bz2', 'gz', 'xz', 'lzo' and a "tar" version of them: 'tar.bz2', 'tbz2', 'tbz',
 'tb2', 'tar.gz', 'tgz', 'tar.xz', 'txz', 'tar.lzo', 'tzo', 'tar.lz4', 'tlz4'.
@@ -25,15 +25,13 @@ gzip, pigz, xz, lzop, lz4, tar and unzip.
 """
 
 import os
+import io
 import errno
 import sys
-if sys.version[0] == '2':
-    import urlparse
-else:
-    import urllib.parse as urlparse
 import logging
 import threading
 import subprocess
+from six.moves.urllib import parse as urlparse
 from bmaptools import BmapHelpers
 
 _log = logging.getLogger(__name__)  # pylint: disable=C0103
@@ -167,11 +165,11 @@ class TransRead(object):
         #
         # For example, when the path is an URL to a bz2 file, the chain of
         # opened file will be:
-        #   o self._f_objs[0] is the liburl2 file-like object
+        #   o self._f_objs[0] is the urllib2 file-like object
         #   o self._f_objs[1] is the stdout of the 'bzip2' process
         self._f_objs = []
 
-        self._force_fake_seek = False
+        self._fake_seek = False
         self._pos = 0
 
         try:
@@ -202,7 +200,7 @@ class TransRead(object):
         """
         This function is used when reading compressed files. It runs in a
         spearate thread, reads data from the 'f_from' file-like object, and
-        writes them to the 'f_to' file-like object. 'F_from' may be a urllib2
+        writes them to the 'f_to' file-like object. 'F_from' may be a urllib
         object, while 'f_to' is usually stdin of the decompressor process.
         """
 
@@ -380,10 +378,10 @@ class TransRead(object):
         else:
             args = decompressor + " " + args
 
-        if hasattr(self._f_objs[-1], 'fileno'):
-            child_stdin = self._f_objs[-1].fileno()
-        else:
+        if self.is_url:
             child_stdin = subprocess.PIPE
+        else:
+            child_stdin = self._f_objs[-1].fileno()
 
         child_process = subprocess.Popen(args, shell=True,
                                          bufsize=1024 * 1024,
@@ -392,12 +390,14 @@ class TransRead(object):
                                          stderr=subprocess.PIPE)
 
         if child_stdin == subprocess.PIPE:
+            # A separate reader thread is created only when we are reading via
+            # urllib2.
             args = (self._f_objs[-1], child_process.stdin, )
             self._rthread = threading.Thread(target=self._read_thread, args=args)
             self._rthread.daemon = True
             self._rthread.start()
 
-        self._force_fake_seek = True
+        self._fake_seek = True
         self._f_objs.append(child_process.stdout)
         self._child_processes.append(child_process)
 
@@ -475,7 +475,7 @@ class TransRead(object):
 
         self._child_processes.append(child_process)
         self.is_url = True
-        self._force_fake_seek = True
+        self._fake_seek = True
 
     def _open_url(self, url):
         """
@@ -493,17 +493,14 @@ class TransRead(object):
 
         import socket
 
-        if sys.version[0] == '2':
-            import httplib
-            import urllib2
-        else:
-            import http.client as httplib
-            import urllib.request as urllib2
+        from six.moves import http_client as httplib
+        from six.moves.urllib import request as urllib
+        from six.moves.urllib.error import URLError
 
         parsed_url = urlparse.urlparse(url)
 
         if parsed_url.scheme == "ssh":
-            # Unfortunately, liburl2 does not handle "ssh://" URLs
+            # Unfortunately, urllib2 does not handle "ssh://" URLs
             self._open_url_ssh(parsed_url)
             return
 
@@ -522,15 +519,15 @@ class TransRead(object):
             url = urlparse.urlunparse(new_url)
 
             # Build an URL opener which will do the authentication
-            password_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
+            password_manager = urllib.HTTPPasswordMgrWithDefaultRealm()
             password_manager.add_password(None, url, username, password)
-            auth_handler = urllib2.HTTPBasicAuthHandler(password_manager)
-            opener = urllib2.build_opener(auth_handler)
+            auth_handler = urllib.HTTPBasicAuthHandler(password_manager)
+            opener = urllib.build_opener(auth_handler)
         else:
-            opener = urllib2.build_opener()
+            opener = urllib.build_opener()
 
         opener.addheaders = [('User-Agent', 'Mozilla/5.0')]
-        urllib2.install_opener(opener)
+        urllib.install_opener(opener)
 
         # Open the URL. First try with a short timeout, and print a message
         # which should supposedly give the a clue that something may be going
@@ -549,7 +546,7 @@ class TransRead(object):
                     _print_warning(timeout)
                 else:
                     raise Error("cannot open URL '%s': %s" % (url, err))
-            except urllib2.URLError as err:
+            except URLError as err:
                 # Handling the timeout case in Python 2.6
                 if timeout is not None and \
                    isinstance(err.reason, socket.timeout):
@@ -580,15 +577,20 @@ class TransRead(object):
 
     def seek(self, offset, whence=os.SEEK_SET):
         """The 'seek()' method, similar to the one file objects have."""
-        if self._force_fake_seek or not hasattr(self._f_objs[-1], "seek"):
+        if self._fake_seek or not hasattr(self._f_objs[-1], "seek"):
             self._pos = _fake_seek_forward(self._f_objs[-1], self._pos,
                                            offset, whence)
         else:
-            self._f_objs[-1].seek(offset, whence)
+            try:
+                self._f_objs[-1].seek(offset, whence)
+            except io.UnsupportedOperation:
+                self._fake_seek = True
+                self._pos = _fake_seek_forward(self._f_objs[-1], self._pos,
+                                               offset, whence)
 
     def tell(self):
         """The 'tell()' method, similar to the one file objects have."""
-        if self._force_fake_seek or not hasattr(self._f_objs[-1], "tell"):
+        if self._fake_seek or not hasattr(self._f_objs[-1], "tell"):
             return self._pos
         else:
             return self._f_objs[-1].tell()
